@@ -47,6 +47,7 @@ from rattail.db.model import (
     Product, ProductPrice, ProductCost, ProductCode,
     Brand, Vendor, Department, Subdepartment, LabelProfile)
 from rattail.gpc import GPC
+from rattail.db.api import get_product_by_upc
 
 from ..db import Session
 from ..forms import AutocompleteFieldRenderer, GPCFieldRenderer, PriceFieldRenderer
@@ -72,11 +73,6 @@ class ProductsGrid(SearchableAlchemyGridView):
         #     q = q.outerjoin(Vendor)
         #     return q
 
-        def join_vendor_any(q):
-            return q.outerjoin(ProductCost,
-                               ProductCost.product_uuid == Product.uuid)\
-                .outerjoin(Vendor)
-
         return {
             'brand':
                 lambda q: q.outerjoin(Brand),
@@ -95,7 +91,7 @@ class ProductsGrid(SearchableAlchemyGridView):
             # 'vendor':
             #     join_vendor,
             'vendor_any':
-                join_vendor_any,
+                lambda q: q.outerjoin(ProductCost, ProductCost.product_uuid == Product.uuid).outerjoin(Vendor),
             'code':
                 lambda q: q.outerjoin(ProductCode),
             }
@@ -250,6 +246,30 @@ class ProductCrud(CrudView):
         return fs
 
 
+def products_search(request):
+    """
+    Locate a product(s) by UPC.
+
+    Eventually this should be more generic, or at least offer more fields for
+    search.  For now it operates only on the ``Product.upc`` field.
+    """
+
+    product = None
+    upc = request.GET.get('upc')
+    if upc:
+        product = get_product_by_upc(Session, upc)
+        if not product:
+            # Try again, assuming caller did not include check digit.
+            upc = GPC(upc, calc_check_digit='upc')
+            product = get_product_by_upc(Session, upc)
+        if product:
+            product = {
+                'uuid':                 product.uuid,
+                'full_description':     product.full_description,
+                }
+    return {'product': product}
+
+
 def print_labels(request):
     profile = request.params.get('profile')
     profile = Session.query(LabelProfile).get(profile) if profile else None
@@ -346,6 +366,7 @@ class CreateProductsBatch(ProductsGrid):
 
 def add_routes(config):
     config.add_route('products', '/products')
+    config.add_route('products.search', '/products/search')
     config.add_route('products.print_labels', '/products/labels')
     config.add_route('products.create_batch', '/products/batch')
     config.add_route('product.create', '/products/new')
@@ -376,3 +397,5 @@ def includeme(config):
                     permission='products.update')
     config.add_view(ProductCrud, attr='delete', route_name='product.delete',
                     permission='products.delete')
+    config.add_view(products_search, route_name='products.search',
+                    renderer='json', permission='products.list')
